(function() {
  var CRLF, Channel, Client, JsonLineProtocol, argv, channels, clients, drop_client, handle_request, log, net, opt, part, server, started_at, util;
  opt = require('optimist');
  argv = opt.options('host', {
    alias: 'h',
    "default": '127.0.0.1'
  }).options('port', {
    alias: 'p',
    "default": 11746
  }).options('log-level', {
    alias: 'l',
    "default": 'error'
  }).options('max-input', {
    alias: 'M',
    "default": 1024 * 1024
  }).argv;
  net = require('net');
  util = require('util');
  log = new (require('log'))(argv['log-level'], process.stderr);
  JsonLineProtocol = (require('json-line-protocol')).JsonLineProtocol;
  CRLF = '\r\n';
  Client = (function() {
    function Client(stream) {
      this.stream = stream;
      this.nick = null;
      this.channels = {};
      this.stream.setEncoding('utf8');
      this.stream.setTimeout(0);
      this.stream.setKeepAlive(true);
    }
    Client.prototype.join = function(name) {
      return this.channels[name] = true;
    };
    Client.prototype.part = function(name) {
      return delete this.channels[name];
    };
    return Client;
  })();
  Channel = (function() {
    function Channel(name) {
      this.name = name;
      this.clients = {};
    }
    Channel.prototype.join = function(client) {
      if (!this.clients[client.nick]) {
        this.broadcast(client, {
          join: this.name,
          who: client.nick
        });
        return this.clients[client.nick] = client;
      }
    };
    Channel.prototype.part = function(client, why) {
      if (this.clients[client.nick] != null) {
        this.broadcast(client, {
          part: this.name,
          who: client.nick,
          why: why
        });
        return delete this.clients[client.nick];
      }
    };
    Channel.prototype.say = function(client, what) {
      return this.broadcast(client, {
        say: this.name,
        who: client.nick,
        what: what
      });
    };
    Channel.prototype.change_nick = function(client, old_nick, new_nick) {
      delete this.clients[old_nick];
      this.clients[new_nick] = client;
      return this.broadcast(client, {
        nick: this.name,
        old_nick: old_nick,
        new_nick: new_nick
      });
    };
    Channel.prototype.broadcast = function(sender, object) {
      var client, json, _, _ref, _results;
      json = JSON.stringify(object);
      _ref = this.clients;
      _results = [];
      for (_ in _ref) {
        client = _ref[_];
        _results.push(client !== sender ? client.stream.write(json + CRLF) : void 0);
      }
      return _results;
    };
    return Channel;
  })();
  clients = {};
  channels = {};
  started_at = Date.now();
  part = function(client, channel_name, why) {
    if (client.channels[channel_name] != null) {
      client.part(channel_name);
      channels[channel_name].part(client, why != null ? why : '');
      if (channels[channel_name].clients.length === 0) {
        delete channels[channel_name];
      }
    }
  };
  drop_client = function(client, why) {
    var channel_name, _, _ref, _ref2;
    log.info("dropping client " + ((_ref = client.nick) != null ? _ref : '<no-nickname>') + ": " + why);
    if (client.stream.writable) {
      client.stream.write((JSON.stringify({
        error: why
      })) + CRLF);
    }
    _ref2 = client.channels;
    for (channel_name in _ref2) {
      _ = _ref2[channel_name];
      part(client, channel_name, why);
    }
    client.stream.destroySoon();
    if (client.nick != null) {
      delete clients[client.nick];
    }
  };
  handle_request = function(stream) {
    var client, protocol;
    client = new Client(stream);
    protocol = new JsonLineProtocol(argv['max-input']);
    stream.on('close', function(had_error) {
      return drop_client(client, 'quit');
    });
    stream.on('data', function(data) {
      return protocol.feed(data);
    });
    protocol.on('error', function(error, line) {
      log.error("client protocol error: " + (error.toString()) + " line:" + line);
      return drop_client(client, 'protocol error');
    });
    protocol.on('overflow', function() {
      log.error("client is flooding");
      return drop_client(client, 'flooding');
    });
    return protocol.on('value', function(command) {
      var channel_name, info, matches, message, name, new_nick, nick, nicks, old_nick, prefix, _, _ref, _ref2, _ref3, _ref4, _ref5;
      if (!(client.nick != null) && !(command.nick != null)) {
        drop_client(client, 'set nickname first');
        return;
      }
      if (command.nick != null) {
        _ref = [client.nick, command.nick], old_nick = _ref[0], new_nick = _ref[1];
        if (clients[new_nick] != null) {
          stream.write((JSON.stringify({
            error: 'non-unique nickname'
          })) + CRLF);
          return;
        }
        client.nick = new_nick;
        clients[new_nick] = client;
        if (old_nick != null) {
          _ref2 = client.channels;
          for (channel_name in _ref2) {
            _ = _ref2[channel_name];
            channels[channel_name].change_nick(client, old_nick, new_nick);
          }
          delete clients[old_nick];
        }
        return;
      }
      if (command.list != null) {
        prefix = (_ref3 = command.list) != null ? _ref3 : '';
        matches = (function() {
          var _results;
          _results = [];
          for (name in channels) {
            _ = channels[name];
            if (name.indexOf(prefix === 0)) {
              _results.push(name);
            }
          }
          return _results;
        })();
        stream.write((JSON.stringify({
          list: matches
        })) + CRLF);
        return;
      }
      if (command.join != null) {
        channel_name = command.join;
        if (client.channels[channel_name] == null) {
          (channels[channel_name] || (channels[channel_name] = new Channel(channel_name))).join(client);
          client.join(channel_name);
        }
        return;
      }
      if (command.part != null) {
        channel_name = command.part;
        part(client, channel_name, (_ref4 = command.why) != null ? _ref4 : '');
        return;
      }
      if (command.members != null) {
        channel_name = command.members;
        nicks = channels[channel_name] != null ? (function() {
          var _ref5, _results;
          _ref5 = channels[channel_name].clients;
          _results = [];
          for (nick in _ref5) {
            _ = _ref5[nick];
            _results.push(nick);
          }
          return _results;
        })() : [];
        stream.write((JSON.stringify({
          members: channel_name,
          nicks: nicks
        })) + CRLF);
        return;
      }
      if (command.say != null) {
        channel_name = command.say;
        if (client.channels[channel_name] == null) {
          stream.write((JSON.stringify({
            error: 'non-members cannot chat'
          })) + CRLF);
          drop_client(client, 'protocol error');
          return;
        }
        message = ((_ref5 = command.what) != null ? _ref5 : '').trim();
        if (!(message.length > 0)) {
          stream.write((JSON.stringify({
            error: 'blank message'
          })) + CRLF);
          drop_client(client, 'protocol error');
          return;
        }
        channels[channel_name].say(client, message);
        return;
      }
      if (command.info != null) {
                info = {
          channels: (Object.keys(channels)).length,
          users: (Object.keys(clients)).length,
          uptime: +(Date.now() - started_at) / 1000
        };
        stream.write((JSON.stringify({
          info: info
        })) + CRLF);;
      }
    });
  };
  server = net.createServer(handle_request);
  server.listen(argv.port, argv.host, function() {
    return log.info("ready on " + argv.host + ":" + argv.port);
  });
}).call(this);
