(function() {
  var CRLF, ChatClient, EventEmitter, JsonLineProtocol, net, util;
  var __hasProp = Object.prototype.hasOwnProperty, __extends = function(child, parent) {
    for (var key in parent) { if (__hasProp.call(parent, key)) child[key] = parent[key]; }
    function ctor() { this.constructor = child; }
    ctor.prototype = parent.prototype;
    child.prototype = new ctor;
    child.__super__ = parent.prototype;
    return child;
  }, __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; };
  net = require('net');
  util = require('util');
  EventEmitter = (require('events')).EventEmitter;
  JsonLineProtocol = (require('json-line-protocol')).JsonLineProtocol;
  CRLF = '\r\n';
  ChatClient = (function() {
    __extends(ChatClient, EventEmitter);
    function ChatClient(port, host) {
      this.list_callback = null;
      this.info_callback = null;
      this.members_callbacks = {};
      this.conn = net.createConnection(port, host);
      this.conn.on('connect', __bind(function() {
        this.conn.setEncoding('utf8');
        this.conn.setKeepAlive(true);
        return this.conn.setNoDelay(true);
      }, this));
      this.protocol = new JsonLineProtocol;
      this.conn.on('data', __bind(function(data) {
        return this.protocol.feed(data);
      }, this));
      this.conn.on('error', __bind(function(error) {
        return this.emit('client-error', error.toString());
      }, this));
      this.conn.on('end', __bind(function() {
        return this.emit('client-error', 'connection-closed');
      }, this));
      this.protocol.on('error', __bind(function(error, line) {
        return this.emit('protocol-error', error, line);
      }, this));
      this.protocol.on('value', __bind(function(msg) {
        var channel_name, _base;
        if (msg.error != null) {
          this.emit('server-error', msg.error);
          return;
        }
        if (msg.list != null) {
          if (typeof this.list_callback === "function") {
            this.list_callback(msg.list);
          }
          this.list_callback = null;
          return;
        }
        if (msg.members != null) {
          channel_name = msg.members;
          if (typeof (_base = this.members_callbacks)[channel_name] === "function") {
            _base[channel_name](msg.nicks);
          }
          delete this.members_callbacks[channel_name];
          return;
        }
        if (msg.info != null) {
          if (typeof this.info_callback === "function") {
            this.info_callback(msg.info);
          }
          this.info_callback = null;
          return;
        }
        if (msg.join != null) {
          this.emit('join', msg.join, msg.who);
          return;
        }
        if (msg.part != null) {
          this.emit('part', msg.part, msg.who, msg.why);
          return;
        }
        if (msg.say != null) {
          this.emit('say', msg.say, msg.who, msg.what);
          return;
        }
        if (msg.nick != null) {
          this.emit('nick', msg.nick, msg.old_nick, msg.new_nick);
          return;
        }
        throw new Error("unexpected message " + (util.inspect(msg)));
      }, this));
    }
    ChatClient.prototype.close = function() {
      return this.conn.end();
    };
    ChatClient.prototype._write = function(obj) {
      var json;
      json = JSON.stringify(obj);
      if (this.conn.writable) {
        return this.conn.write(json + CRLF);
      }
    };
    ChatClient.prototype.nick = function(nick) {
      return this._write({
        nick: nick
      });
    };
    ChatClient.prototype.list_filter = function(prefix, callback) {
      this.list_callback = callback;
      return this._write({
        list: prefix
      });
    };
    ChatClient.prototype.list = function(callback) {
      this.list_callback = callback;
      return this._write({
        list: true
      });
    };
    ChatClient.prototype.join = function(channel) {
      return this._write({
        join: channel
      });
    };
    ChatClient.prototype.members = function(channel, callback) {
      this.members_callbacks[channel] = callback;
      return this._write({
        members: channel
      });
    };
    ChatClient.prototype.say = function(channel, what) {
      return this._write({
        say: channel,
        what: what
      });
    };
    ChatClient.prototype.part = function(channel, why) {
      if (why == null) {
        why = '';
      }
      return this._write({
        part: channel,
        why: why
      });
    };
    ChatClient.prototype.info = function(callback) {
      this.info_callback = callback;
      return this._write({
        info: true
      });
    };
    return ChatClient;
  })();
  exports.connect = function(port, host) {
    if (port == null) {
      port = 11746;
    }
    if (host == null) {
      host = '127.0.0.1';
    }
    return new ChatClient(port, host);
  };
}).call(this);
